/*
 * xrick/src/e_bomb.c
 *
 * This source code file is part of the xrick distribution, and as such is
 * copyrighted. Please read the README file in the root directory of the
 * distribution for information about copyrights, trademarks, license,
 * disclaimers, etc.
 */

#include "system.h"
#include "global.h"
#include "ents.h"
#include "e_bomb.h"

#include "e_rick.h"
#include "sound.h"

/*
 * public vars
 * public for speed reasons: could be private and accessed via functions
 */
U8 e_bomb_ticker;
U8 e_bomb_lethal;
U8 e_bomb_xc;
U16 e_bomb_yc;


/*
 * Bomb hit test
 *
 * ASM 11CD
 * returns: G_TRUE/hit, G_FALSE/not
 */
U8
e_bomb_hit(U8 e)
{
  if (ent_ents[e].x >
      (E_BOMB_ENT.x >= 0xE0 ? 0xFF : E_BOMB_ENT.x + 0x20))
    return G_FALSE;
  if (ent_ents[e].x + ent_ents[e].w < 
      (E_BOMB_ENT.x > 0x04 ? E_BOMB_ENT.x - 0x04 : 0))
    return G_FALSE;
  if (ent_ents[e].y >
      (E_BOMB_ENT.y + 0x1D))
    return G_FALSE;
  if (ent_ents[e].y + ent_ents[e].h <
      (E_BOMB_ENT.y > 0x0004 ? E_BOMB_ENT.y - 0x0004 : 0))
    return G_FALSE;
  return G_TRUE;
}


/*
 * Entity action
 *
 * ASM 18CA
 */
void
e_bomb_action(U8 e) /* warning: unused parameter 'e' */
{
  static U8 sp[] = {0x24, 0x25, 0x26, 0x27, 0x28};  /* sprites sequence */

  /* tick */
  e_bomb_ticker--;

  if (!e_bomb_ticker) {
    /* if end then deactivate */
    E_BOMB_ENT.n = 0;
    e_bomb_lethal = G_FALSE;
  }

  if (e_bomb_ticker >= 0x0A) {
    /* if still ticking */
    if (e_bomb_ticker == 0x0B)
      snd_something();
    E_BOMB_ENT.sprite = (e_bomb_ticker & 0x01) ? 0x23 : 0x22;
    e_bomb_lethal = G_FALSE;
  }
  else {
    /* explode */
    e_bomb_xc = E_BOMB_ENT.x + 0x0C;
    e_bomb_yc = E_BOMB_ENT.y + 0x000A;
    e_bomb_lethal = G_TRUE;
    E_BOMB_ENT.sprite = sp[e_bomb_ticker >> 1];

    if (e_bomb_hit(E_RICK_NO))
      e_rick_gozombie();

  }
}

/* eof */


