/*
 * xrick/src/scr_imain.c
 *
 * This source code file is part of the xrick distribution, and as such is
 * copyrighted. Please read the README file in the root directory of the
 * distribution for information about copyrights, trademarks, license,
 * disclaimers, etc.
 */

#include <stddef.h>

#include "system.h"
#include "global.h"
#include "screens.h"

#include "video.h"
#include "draw.h"
#include "kbd.h"
#include "pics.h"

/*
 * Main introduction
 *
 * return: 0 while running, 1 when finished.
 */
U8
scr_imain(void)
{
  static U8 seq = 0;
  static U8 first = 0xff;
  static U8 speriod = 0;
  static U32 tm = 0;

  if (seq == 0) {
    d_tlbank = 0;
    seq = 1;
    speriod = g_period;
    g_period = 50;
  }

  switch (seq) {
  case 1:  /* dispay hall of fame */
    v_clear();
    tm = sys_gettime();

    /* hall of fame title */
#ifdef GFXPC
    d_tllst = (U8 *)scr_imainhoft;
    d_setfbp(32, 0);
    d_filter = 0xaaaa;
    d_tl_lst();
#endif
#ifdef GFXST
    d_pic(0, 0, 0x140, 0x20, pic_haf);
#endif

    /* hall of fame content */
    d_tllst = (U8 *)scr_imainhof;
    d_setfbp(56, 40);
    d_filter = 0x5555;
    d_tl_lst();

    g_rects = &v_fbrect;
    seq = 2;
    break;

  case 2:  /* wait for key pressed or timeout */
    g_rects = NULL;
    if (k_status & K_FIRE)
      seq = 3;
    else if (sys_gettime() - tm > SCR_TMOUT)
      seq = 4;
    break;

  case 3:  /* wait for key released */
    if (!(k_status & K_FIRE)) {
      if (first)
	seq = 4;
      else
	seq = 7;
    }
    break;

  case 4:  /* display Rick Dangerous title and Core Design copyright */
    v_clear();
    tm = sys_gettime();

#ifdef GFXPC
    /* Rick Dangerous title */
    d_tllst = (U8 *)scr_imainrdt;
    d_setfbp(32, 16);
    d_filter = 0xaaaa;
    d_tl_lst();
    
    /* Core Design copyright + press space to start */
    d_tllst = (U8 *)scr_imaincdc;
    d_setfbp(64, 80);
    d_filter = 0x5555;
    d_tl_lst();
#endif

#ifdef GFXST
    d_pic(0, 0, 0x140, 0xc8, pic_splash);
#endif
    
    g_rects = &v_fbrect;
    seq = 5;
    first = 0;
    break;

  case 5:  /* wait for key pressed or timeout */
    g_rects = NULL;
    if (k_status & K_FIRE)
      seq = 6;
    else if (sys_gettime() - tm > SCR_TMOUT)
      seq = 1;
    break;

  case 6:  /* wait for key released */
    if (!(k_status & K_FIRE))
      seq = 7;
    break;
  }

  if (seq == 7) {  /* we're done */
    v_clear();
    g_rects = &v_fbrect;
    seq = 0;
    g_period = speriod;
    /*sys_printf("xrick/screens: end main intro\n");*/
    return 1;
  }
  else
    return 0;
}

/* eof */


