/*
 * xrick/src/sys_linux.c
 *
 * This source code file is part of the xrick distribution, and as such is
 * copyrighted. Please read the README file in the root directory of the
 * distribution for information about copyrights, trademarks, license,
 * disclaimers, etc.
 */

#include "SDL.h"

#include <stdarg.h>   /* args for panic */
#include <fcntl.h>    /* fcntl in panic */
#include <stdio.h>    /* printf */
#include <sys/time.h> /* timer */
#include <unistd.h>   /* timer */

#include "system.h"
#include "e_rick.h"
#include "parms.h"
#include "mainloop.h"

/*
 * panic
 *
 */
void
panic(char *err, ...)
{
  va_list argptr;
  char s[1024];

  /* change stdin to non blocking */
  fcntl(0, F_SETFL, fcntl (0, F_GETFL, 0) & ~FNDELAY);
  /* NOTE HPUX: use ... is it OK on Linux ? */
  /* fcntl(0, F_SETFL, fcntl (0, F_GETFL, 0) & ~O_NDELAY); */

  /* prepare message */
  va_start(argptr, err);
  vsprintf(s, err, argptr);
  va_end(argptr);

  /* print message and die */
  printf("%s\npanic!\n", s);
  exit(1);
}


/*
 * sys_printf
 */
#ifdef DEBUG
void
sys_printf(char *msg, ...)
{
  va_list argptr;
  char s[1024];

  /* change stdin to non blocking */
  /*fcntl(0, F_SETFL, fcntl (0, F_GETFL, 0) & ~FNDELAY);*/
  /* NOTE HPUX: use ... is it OK on Linux ? */
  /* fcntl(0, F_SETFL, fcntl (0, F_GETFL, 0) & ~O_NDELAY); */

  /* prepare message */
  va_start(argptr, msg);
  vsprintf(s, msg, argptr);
  va_end(argptr);
  printf(s);
}
#endif


/*
 * sys_gettime
 *
 */
U32
sys_gettime(void)
{
  static U32 ticks_base = 0;
  U32 ticks;

  ticks = SDL_GetTicks();

  if (!ticks_base)
    ticks_base = ticks;

  return ticks - ticks_base;
}


/*
 * sys_sleep(int);
 *
 */
void
sys_sleep(int s)
{
  SDL_Delay(s);
}


/*
 * main
 */
int
main(int argc, char *argv[])
{
  int f;

  /* process args */
  if (p_init(argc, argv) == -1)
    panic("xrick/panic: bad args\n");

  /* FIXME this will disappear */
  /*
  f = open("data/ds.bin", O_RDONLY);
  read(f, ds, 65535);
  close(f);
  */

  /* FIXME */
#ifdef DEBUG
  printf("sizeof U8 is %d\n", sizeof(U8));
  printf("sizeof U16 is %d\n", sizeof(U16));
#endif

  /* init everything */
  v_init();
#ifdef GFXPC
  spr_init();
#endif

  /* play */
  mainloop();

  /* shutdown everything */
  v_shutdown();

  return 0;
}


/* eof */
