/*
 * xrick/src/vid_sdl.c
 *
 * This source code file is part of the xrick distribution, and as such is
 * copyrighted. Please read the README file in the root directory of the
 * distribution for information about copyrights, trademarks, license,
 * disclaimers, etc.
 */

#include "SDL.h"

#include "system.h"
#include "video.h"
#include "videoex.h"

#include "global.h"
#include "e_rick.h"
#include "rects.h"
#include "parms.h"
#include "kbd.h"

U8 *v_fb;               /* frame buffer */
U8 v_zoom = V_ZOOM, v_zoom0;              /* zoom */
rect_t v_fbrect = {0, 0, V_WIDTH, V_HEIGHT, NULL}; /* whole frame buffer */

SDL_Surface *screen;
U32 videoflags;
SDL_Event event;

/*
 * color tables
 */

#ifdef GFXPC
static U8 RED[] = { 0x00, 0x50, 0xf0, 0xf0, 0x00, 0x50, 0xf0, 0xf0 };
static U8 GREEN[] = { 0x00, 0xf8, 0x50, 0xf8, 0x00, 0xf8, 0x50, 0xf8 };
static U8 BLUE[] = { 0x00, 0x50, 0x50, 0x50, 0x00, 0xf8, 0xf8, 0xf8 };
#endif
#ifdef GFXST
static U8 RED[] = { 0x00, 0xd8, 0xb0, 0xf8,
                    0x20, 0x00, 0x00, 0x20,
                    0x48, 0x48, 0x90, 0xd8,
                    0x48, 0x68, 0x90, 0xb0,
                    /* cheat colors */
                    0x50, 0xe0, 0xc8, 0xf8,
                    0x68, 0x50, 0x50, 0x68,
                    0x80, 0x80, 0xb0, 0xe0,
                    0x80, 0x98, 0xb0, 0xc8
};
static U8 GREEN[] = { 0x00, 0x00, 0x6c, 0x90,
                      0x24, 0x48, 0x6c, 0x48,
                      0x6c, 0x24, 0x48, 0x6c,
                      0x48, 0x6c, 0x90, 0xb4,
		      /* cheat colors */
                      0x54, 0x54, 0x9c, 0xb4,
                      0x6c, 0x84, 0x9c, 0x84,
                      0x9c, 0x6c, 0x84, 0x9c,
                      0x84, 0x9c, 0xb4, 0xcc
};
static U8 BLUE[] = { 0x00, 0x00, 0x68, 0x68,
                     0x20, 0xb0, 0xd8, 0x00,
                     0x20, 0x00, 0x00, 0x00,
                     0x48, 0x68, 0x90, 0xb0,
		     /* cheat colors */
                     0x50, 0x50, 0x98, 0x98,
                     0x68, 0xc8, 0xe0, 0x50,
                     0x68, 0x50, 0x50, 0x50,
                     0x80, 0x98, 0xb0, 0xc8};
#endif


/*
 * 
 */
SDL_Surface *v_initscreen(U16 w, U16 h, U8 bpp, U32 flags)
{
  SDL_Surface *scr;
  SDL_Color pal[256];
  U8 i;

  scr = SDL_SetVideoMode(w, h, bpp, flags);
  if (!scr)
    return scr;
  for (i = 0; i < 32; ++i) {
    pal[i].r = RED[i];
    pal[i].g = GREEN[i];
    pal[i].b = BLUE[i];
  }
  SDL_SetColors(scr, pal, 0, 32);
  return scr;
}


/*
 * Initialise video
 */
void
v_init(void)
{
#ifdef DEBUG
  printf("xrick/video: start\n");
#endif

  /* SDL */
  if (SDL_Init(SDL_INIT_VIDEO|SDL_INIT_TIMER) < 0)
    panic("xrick/video: could not init SDL\n");
  videoflags = SDL_HWSURFACE|SDL_HWPALETTE;
  screen = v_initscreen(V_WIDTH * v_zoom, V_HEIGHT * v_zoom, 8, videoflags);

  SDL_WM_SetCaption("xrick", "xrick");

  /*
   * create v_ frame buffer
   */
  v_fb = malloc(V_WIDTH * V_HEIGHT);
  if (!v_fb)
    panic("xrick/video: v_fb malloc failed\n");

#ifdef DEBUG
  printf("xrick/video: ready\n");
#endif
}


/*
 * v_shutdown
 */
void
v_shutdown(void)
{
#ifdef DEBUG
  printf("xrick/video: shutdown\n");
#endif

  free(v_fb);
  v_fb = NULL;

  SDL_Quit();

#ifdef DEBUG
  printf("xrick/video: down\n");
#endif
}



/*
 * v_update
 * NOTE errors processing ?
 */
void
v_update(rect_t *rects)
{
  SDL_Rect area;
  U16 x, y, xz, yz;
  U8 *p, *q, *p0, *q0;

  while (rects) {

    p0 = v_fb;
    p0 += rects->x + rects->y * V_WIDTH;
    q0 = (U8 *)screen->pixels;
    q0 += (rects->x + rects->y * V_WIDTH * v_zoom) * v_zoom;

    for (y = rects->y; y < rects->y + rects->height; y++) {
      for (yz = 0; yz < v_zoom; yz++) {
	p = p0;
	q = q0;
	for (x = rects->x; x < rects->x + rects->width; x++) {
	  for (xz = 0; xz < v_zoom; xz++) {
	    *q = *p;
	    q++;
	  }
	  p++;
	}
	q0 += V_WIDTH * v_zoom;
      }
      p0 += V_WIDTH;
    }

    area.x = rects->x * v_zoom;
    area.y = rects->y * v_zoom;
    area.h = rects->height * v_zoom;
    area.w = rects->width * v_zoom;
    SDL_UpdateRects(screen, 1, &area);

    rects = rects->next;
  }
}


/*
 * v_clear (077C)
 */
void
v_clear(void)
{
  memset(v_fb, 0, V_WIDTH * V_HEIGHT);
}


static U8 v_kbd_up = SDLK_o;
static U8 v_kbd_down = SDLK_k;
static U8 v_kbd_left = SDLK_z;
static U8 v_kbd_right = SDLK_x;
static U8 v_kbd_pause = SDLK_p;
static U8 v_kbd_end = SDLK_e;
static U8 v_kbd_xtra = SDLK_ESCAPE;
static U8 v_kbd_fire = SDLK_SPACE;


/*
 * v_event
 */
static U8
v_event()
{
  U8 keyevent = 0;
  U8 key;

  switch (event.type) {
  case SDL_KEYDOWN:
    key = event.key.keysym.sym;
    if (key == v_kbd_up) {
      k_status |= K_UP;
      k_last = K_UP;
      keyevent = 0xFF;
    }
    else if (key == v_kbd_down) {
      k_status |= K_DOWN;
      k_last = K_DOWN;
      keyevent = 0xFF;
    }
    else if (key == v_kbd_left) {
      k_status |= K_LEFT;
      k_last = K_LEFT;
      keyevent = 0xFF;
    }
    else if (key == v_kbd_right) {
      k_status |= K_RIGHT;
      k_last = K_RIGHT;
      keyevent = 0xFF;
    }
    else if (key == v_kbd_pause) {
      k_status |= K_PAUSE;
      k_last = K_PAUSE;
      keyevent = 0xFF;
    }
    else if (key == v_kbd_end) {
      k_status |= K_END;
      k_last = K_END;
      keyevent = 0xFF;
    }
    else if (key == v_kbd_xtra) {
      k_status |= K_XTRA;
      k_last = K_XTRA;
      keyevent = 0xFF;
    }
    else if (key == v_kbd_fire) {
      k_status |= K_FIRE;
      k_last = K_FIRE;
      keyevent = 0xFF;
    }
    else if (key == SDLK_1) {
      videoflags ^= SDL_FULLSCREEN;
      if (videoflags & SDL_FULLSCREEN) {
	v_zoom0 = v_zoom;
	v_zoom = 1;
      }
      else {
	v_zoom = v_zoom0;
      }
      screen = v_initscreen(V_WIDTH * v_zoom,
			    V_HEIGHT * v_zoom,
			    screen->format->BitsPerPixel, videoflags);
      v_update(&v_fbrect);
    }
    else if (key == SDLK_2 && v_zoom > 1) {
      if (videoflags & SDL_FULLSCREEN) {
	v_zoom0 -= 1;
      }
      else {
	v_zoom -= 1;
	screen = v_initscreen(V_WIDTH * v_zoom,
			      V_HEIGHT * v_zoom,
			      screen->format->BitsPerPixel, videoflags);
	v_update(&v_fbrect);
      }
    }
    else if (key == SDLK_3 && v_zoom < 4) {
      if (videoflags & SDL_FULLSCREEN) {
	v_zoom0 += 1;
      }
      else {
	v_zoom += 1;
	screen = v_initscreen(V_WIDTH * v_zoom,
			      V_HEIGHT * v_zoom,
			      screen->format->BitsPerPixel, videoflags);
	v_update(&v_fbrect);
      }
    }
    else if (key == SDLK_7) {
      g_cheat1 = ~g_cheat1;
      g_lives = 6;
      g_bombs = 6;
      g_bullets = 6;
      d_status();
      v_update(&v_fbrect);
    }
    else if (key == SDLK_8) {
      g_cheat2 = ~g_cheat2;
      d_status();
      v_update(&v_fbrect);
    }
    else if (key == SDLK_9) {
      g_cheat3 = ~g_cheat3;
      d_status();
      v_update(&v_fbrect);
    }
    break;
  case SDL_KEYUP:
    key = event.key.keysym.sym;
    if (key == v_kbd_up) {
      k_status ^= K_UP;
      k_last = K_UP;
      keyevent = 0xFF;
    }
    else if (key == v_kbd_down) {
      k_status ^= K_DOWN;
      k_last = K_DOWN;
      keyevent = 0xFF;
    }
    else if (key == v_kbd_left) {
      k_status ^= K_LEFT;
      k_last = K_LEFT;
      keyevent = 0xFF;
    }
    else if (key == v_kbd_right) {
      k_status ^= K_RIGHT;
      k_last = K_RIGHT;
      keyevent = 0xFF;
    }
    else if (key == v_kbd_pause) {
      k_status ^= K_PAUSE;
      k_last = K_PAUSE;
      keyevent = 0xFF;
    }
    else if (key == v_kbd_end) {
      k_status ^= K_END;
      k_last = K_END;
      keyevent = 0xFF;
    }
    else if (key == v_kbd_xtra) {
      k_status ^= K_XTRA;
      k_last = K_XTRA;
      keyevent = 0xFF;
    }
    else if (key == v_kbd_fire) {
      k_status ^= K_FIRE;
      k_last = K_FIRE;
      keyevent = 0xFF;
    }
    break;
  case SDL_QUIT:
    /* FIXME should do something */
    break;
  default:
    break;
  }

  return keyevent;
}
  

/*
 * v_events
 */
void
v_events(void)
{
  while (SDL_PollEvent(&event))
    v_event();
}


/*
 * v_eventsk
 * FIXME ugly 'cos we loop while waiting for events -> remove the XPending!
 */
void
v_eventsk(void)
{
  U8 keyevent = 0;

  do {
    SDL_WaitEvent(&event);
    keyevent |= v_event();
  } while (!keyevent);
}

/* eof */



